setMethod("initialize", "SnpCallSet",
          function(.Object,
                   calls=new("matrix"),
                   callsConfidence=matrix(numeric(), nrow=nrow(calls),
		   ncol=ncol(calls),
		   dimnames=dimnames(calls)), ... ){
            callNextMethod(.Object,
                           calls=calls,
                           callsConfidence=callsConfidence, ...)
          })

setMethod("initialize", "SnpCallSetPlus",
          function(.Object,
                   phenoData, featureData,
                   calls=new("matrix"),
                   callsConfidence=new("matrix"),
                   antisenseThetaA=new("matrix"),
                   antisenseThetaB=new("matrix"),
                   senseThetaA=new("matrix"),
                   senseThetaB=new("matrix"), ... ){
            ad <- assayDataNew("lockedEnvironment",
                               calls=calls,
                               callsConfidence=callsConfidence,
                               antisenseThetaA=antisenseThetaA,
                               antisenseThetaB=antisenseThetaB,
                               senseThetaA=senseThetaA,
                               senseThetaB=senseThetaB)
            
            assayData(.Object) <- ad
            if (missing(phenoData))
              phenoData(.Object) <- annotatedDataFrameFrom(calls, byrow=FALSE)
            if (missing(featureData))
              featureData(.Object) <- annotatedDataFrameFrom(calls, byrow=TRUE)
            .Object
          })

setMethod("initialize", "SnpCnvCallSetPlus",
          function(.Object,
                   phenoData, featureData,
                   calls=new("matrix"),
                   callsConfidence=new("matrix"),
                   antisenseThetaA,
                   antisenseThetaB,
                   senseThetaA,
                   senseThetaB,
                   thetaA,
                   thetaB, ... ){
            ad <- assayDataNew("lockedEnvironment",
                               calls=calls,
                               callsConfidence=callsConfidence,
                               thetaA=thetaA,
                               thetaB=thetaB)
            assayData(.Object) <- ad
            if (missing(phenoData))
              phenoData(.Object) <- annotatedDataFrameFrom(calls, byrow=FALSE)
            if (missing(featureData))
              featureData(.Object) <- annotatedDataFrameFrom(calls, byrow=TRUE)
            .Object
          })



setMethod("initialize", "SnpCopyNumberSet",
          function(.Object,
                   copyNumber=new("matrix"),
                   cnConfidence=matrix(numeric(), nrow=nrow(calls),
                     ncol=ncol(calls),
                     dimnames=dimnames(calls)), ... ){
            callNextMethod(.Object,
                           copyNumber=copyNumber,
                           cnConfidence=cnConfidence, ...)
          })

setMethod("initialize", "oligoSnpSet",
          function(.Object,
                   calls=new("matrix"),
                   callsConfidence=matrix(numeric(), nrow=nrow(calls),
                     ncol=ncol(calls),
                     dimnames=dimnames(calls)),
                   copyNumber=matrix(numeric(), nrow=nrow(calls),
                     ncol=ncol(calls),
                     dimnames=dimnames(calls)),
                   cnConfidence=matrix(numeric(), nrow=nrow(calls),
                     ncol=ncol(calls),
                     dimnames=dimnames(calls)), ... ){
            callNextMethod(.Object,
                           calls=calls,
                           callsConfidence=callsConfidence,
                           copyNumber=copyNumber,
                           cnConfidence=cnConfidence, ...)
          })

setValidity("SnpCallSet", function(object) {
  assayDataValidMembers(assayData(object), c("calls", "callsConfidence"))
})

setValidity("SnpCopyNumberSet", function(object) {
  assayDataValidMembers(assayData(object), c("copyNumber", "cnConfidence"))
})

setValidity("oligoSnpSet", function(object) {
  assayDataValidMembers(assayData(object), c("calls", "callsConfidence", "copyNumber", "cnConfidence"))
})

